{% include 'django/forms/widgets/input.html' %}{% load i18n %}
{% if related_url %}
    <a href="{{ related_url }}" class="related-lookup" id="lookup_id_{{ widget.name }}" title="{{ link_title }}">{% trans 'Select item' %}</a>
{% endif %}
<span class="item-label">
{% if link_label %}
    {% if link_url %}<a href="{{ link_url }}">{% endif %}
    {{ link_label }}
    {% if link_url %}</a>{% endif %}
{% endif %}
</span>
