{% include "admin/edit_inline/stacked.html" %}
<span id="loci-location-json-url"
      data-url="{% url "admin:django_loci_location_json" "0000" %}"></span>
<span id="loci-location-floorplans-json-url"
      data-url="{% url "admin:django_loci_location_floorplans_json" "0000" %}"></span>
